@@ -5,18 +5,260 @@ use warnings;
 use 5.008;
 
 use Pod::Strip;
-use Parse::RecDescent;
+use Parse::RecDescent 1.967009;
 use Module::ExtractUse::Grammar;
 use Carp;
-use version; our $VERSION=version->new('0.24');
+use version; our $VERSION=version->new('0.31');
+
+# ABSTRACT: Find out what modules are used
 
 #$::RD_TRACE=1;
 #$::RD_HINT=1;
 
+
+
+sub new {
+    my $class=shift;
+    return bless {
+        found=>{},
+        files=>0,
+    },$class;
+}
+
+
+# Regular expression to detect eval
+#  On newer perl, you can use named capture groups and (?&name) for recursive regex
+#  However, it requires perl newer than 5.008 declared as requirement in this module
+my $re_block;
+$re_block = qr {
+    ( # eval BLOCK, corresponding to the group 10 in the entire regex
+        {
+            ((?:
+                (?> [^{}]+ )  # Non-braces without backtracking
+            |
+                (??{$re_block}) # Recurse to group 10
+            )*)
+        }
+    )
+}xs;
+my $re = qr{
+    \G(.*?) # group 1
+    eval
+    (?:
+        (?:\s+
+            (?:
+                qq?\((.*?)\) # eval q(), group 2
+                |
+                qq?\[(.*?)\] # eval q[], group 3
+                |
+                qq?{(.*?)}   # eval q{}, group 4
+                |
+                qq?<(.*?)>   # eval q<>, group 5
+                |
+                qq?(\S)(.*?)\6 # eval q'' or so, group 6, group 7
+            )
+        )
+        |
+        (?:\s*(?:
+            (?:(['"])(.*?)\8) # eval '' or eval "", group 8, group 9
+            |
+            ( # eval BLOCK, group 10
+                {
+                    ((?: # group 11
+                        (?> [^{}]+ )  # Non-braces without backtracking
+                    |
+                        (??{$re_block}) # Recurse to group 10
+                    )*)
+                }
+            )
+        ))
+    )
+}xs;
+
+sub extract_use {
+    my $self=shift;
+    my $code_to_parse=shift;
+
+    my $podless;
+    my $pod_parser=Pod::Strip->new;
+    $pod_parser->output_string(\$podless);
+    $pod_parser->parse_characters(1) if $pod_parser->can('parse_characters');
+    if (ref($code_to_parse) eq 'SCALAR') {
+        $pod_parser->parse_string_document($$code_to_parse);
+    }
+    else {
+        $pod_parser->parse_file($code_to_parse);
+    }
+
+    # Strip obvious comments.
+    $podless =~ s/(^|[\};])\s*#.*$/$1/mg;
+
+    my @statements;
+    while($podless =~ /$re/gc) {
+    # to keep parsing time short, split code in statements
+    # (I know that this is not very exact, patches welcome!)
+        my $pre = $1;
+        my $eval = join('', grep { defined $_ } ($2, $3, $4, $5, $7, $9, $11));
+        push @statements, map { [ 0, $_ ] } split(/;/, $pre); # non-eval context
+        push @statements, map { [ 1, $_ ] } split(/;/, $eval); # eval context
+    }
+    push @statements, map { [ 0, $_ ] } split(/;/, substr($podless, pos($podless) || 0)); # non-eval context
+
+    foreach my $statement_ (@statements) {
+        my ($eval, $statement) = @$statement_;
+        $statement=~s/\n+/ /gs;
+        my $result;
+
+        # now that we've got some code containing 'use' or 'require',
+        # parse it! (using different entry point to save some more
+        # time)
+        if ($statement=~/\buse/) {
+            $statement=~s/^(.*?)use\b/use/;
+            eval {
+                my $parser=Module::ExtractUse::Grammar->new();
+                $result=$parser->token_use($statement.';');
+            };
+        }
+        elsif ($statement=~/\brequire/) {
+            $statement=~s/^(.*?)require\b/require/s;
+            eval {
+                my $parser=Module::ExtractUse::Grammar->new();
+                $result=$parser->token_require($statement.';');
+            };
+        }
+
+        next unless $result;
+
+        foreach (split(/\s+/,$result)) {
+            $self->_add($_, $eval) if($_);
+        }
+    }
+
+    # increment file counter
+    $self->_inc_files;
+
+    return $self;
+}
+
+
+
+sub used {
+    my $self=shift;
+    my $key=shift;
+    return $self->{found}{$key} if ($key);
+    return $self->{found};
+}
+
+
+sub used_in_eval {
+    my $self=shift;
+    my $key=shift;
+    return $self->{found_in_eval}{$key} if ($key);
+    return $self->{found_in_eval};
+}
+
+
+sub used_out_of_eval {
+    my $self=shift;
+    my $key=shift;
+    return $self->{found_not_in_eval}{$key} if ($key);
+    return $self->{found_not_in_eval};
+}
+
+
+sub string {
+    my $self=shift;
+    my $sep=shift || ' ';
+    return join($sep,sort keys(%{$self->{found}}));
+}
+
+
+sub string_in_eval {
+    my $self=shift;
+    my $sep=shift || ' ';
+    return join($sep,sort keys(%{$self->{found_in_eval}}));
+}
+
+
+sub string_out_of_eval {
+    my $self=shift;
+    my $sep=shift || ' ';
+    return join($sep,sort keys(%{$self->{found_not_in_eval}}));
+}
+
+
+sub array {
+    return keys(%{shift->{found}})
+}
+
+
+sub array_in_eval {
+    return keys(%{shift->{found_in_eval}})
+}
+
+
+sub array_out_of_eval {
+    return keys(%{shift->{found_not_in_eval}})
+}
+
+
+sub arrayref { 
+    my @a=shift->array;
+    return \@a if @a;
+    return;
+}
+
+
+sub arrayref_in_eval {
+    my @a=shift->array_in_eval;
+    return \@a if @a;
+    return;
+}
+
+
+sub arrayref_out_of_eval {
+    my @a=shift->array_out_of_eval;
+    return \@a if @a;
+    return;
+}
+
+
+sub files {
+    return shift->{files};
+}
+
+# Internal Accessor Methods
+sub _add {
+    my $self=shift;
+    my $found=shift;
+    my $eval=shift;
+    $self->{found}{$found}++;
+    $self->{found_in_eval}{$found}++ if $eval;
+    $self->{found_not_in_eval}{$found}++ unless $eval;
+}
+
+sub _found {
+    return shift->{found}
+}
+
+sub _inc_files {
+    shift->{files}++
+}
+
+1;
+
+__END__
+
+=pod
+
 =head1 NAME
 
 Module::ExtractUse - Find out what modules are used
 
+=head1 VERSION
+
+version 0.31
+
 =head1 SYNOPSIS
 
   use Module::ExtractUse;
@@ -36,6 +278,20 @@ Module::ExtractUse - Find out what modules are used
   
   my @used=$p->array;
   my $used=$p->string;
+  
+  # you can get optional modules, that is use in eval context, in the same style
+  my $used=$p->used_in_eval;           # $used is a HASHREF
+  print $p->used_in_eval('strict')     # true if code includes 'use strict'
+  
+  my @used=$p->array_in_eval;
+  my $used=$p->string_in_eval;
+  
+  # and mandatory modules, that is use out of eval context, in the same style, also.
+  my $used=$p->used_out_of_eval;           # $used is a HASHREF
+  print $p->used_out_of_eval('strict')     # true if code includes 'use strict'
+  
+  my @used=$p->array_out_of_eval;
+  my $used=$p->string_out_of_eval;
 
 =head1 DESCRIPTION
 
@@ -47,26 +303,14 @@ Core, or from CPAN) used by the parsed code.
 
 =head2 Methods
 
-=cut
-
 =head3 new
 
  my $p=Module::ExtractUse->new;
 
 Returns a parser object
 
-=cut
-
-sub new {
-    my $class=shift;
-    return bless {
-        found=>{},
-        files=>0,
-    },$class;
-}
-
 =head3 extract_use
-  
+
   $p->extract_use('/path/to/module.pm');
   $p->extract_use(\$string_containg_code);
 
@@ -78,7 +322,7 @@ $code_to_parse. Or a reference to a SCALAR, in which case
 Module::ExtractUse assumes the referenced scalar contains the source
 code.
 
-The code will be stripped from POD (using Pod::Strip) and splitted on ";"
+The code will be stripped from POD (using Pod::Strip) and split on ";"
 (semicolon). Each statement (i.e. the stuff between two semicolons) is
 checked by a simple regular expression.
 
@@ -90,76 +334,6 @@ you can examine afterwards.
 You can call C<extract_use> several times on different files. It will
 count how many files where examined and how often each module was used.
 
-=cut
-
-sub extract_use {
-    my $self=shift;
-    my $code_to_parse=shift;
-
-    my $podless;
-    my $pod_parser=Pod::Strip->new;
-    $pod_parser->output_string(\$podless);
-    if (ref($code_to_parse) eq 'SCALAR') {
-        $pod_parser->parse_string_document($$code_to_parse);
-    }
-    else {
-        $pod_parser->parse_file($code_to_parse);
-    }
-
-    # Strip obvious comments.
-    $podless =~ s/^\s*#.*$//mg;
-
-    # to keep parsing time short, split code in statements
-    # (I know that this is not very exact, patches welcome!)
-    my @statements=split(/;/,$podless);
-
-    foreach my $statement (@statements) {
-        $statement=~s/\n+/ /gs;
-        my $result;
-
-        # check for string eval in ' ', " " strings
-        if ($statement !~ s/eval\s+(['"])(.*?)\1/$2;/) {
-            # if that didn't work, try q and qq strings
-            if ($statement !~ s/eval\s+qq?(\S)(.*?)\1/$2;/) {
-                # finally try paired delims like qq< >, q( ), ...
-                my %pair = qw| ( ) [ ] { } < > |;
-                while (my ($l, $r) = map {quotemeta} each %pair) {
-                    last if $statement =~ s/eval\s+qq?$l(.*?)$r/$1;/;
-                }
-            }
-        }
-    
-        # now that we've got some code containing 'use' or 'require',
-        # parse it! (using different entry point to save some more
-        # time)
-        if ($statement=~/\buse/) {
-            $statement=~s/^(.*?)use/use/;
-            eval {
-                my $parser=Module::ExtractUse::Grammar->new();
-                $result=$parser->token_use($statement.';');
-            };
-        }
-        elsif ($statement=~/\brequire/) {
-            $statement=~s/^(.*?)require/require/;
-            eval {
-                my $parser=Module::ExtractUse::Grammar->new();
-                $result=$parser->token_require($statement.';');
-            };
-        }
-
-        next unless $result;
-
-        foreach (split(/ /,$result)) {
-            $self->_add($_);
-        }
-    }
-
-    # increment file counter
-    $self->_inc_files;
-
-    return $self;
-}
-
 =head2 Accessor Methods
 
 Those are various ways to get at the result of the parse.
@@ -168,10 +342,8 @@ Note that C<extract_use> returns the parser object, so you can say
 
   print $p->extract_use($code_to_parse)->string;
 
-=cut
-
 =head3 used
-    
+
     my $used=$p->used;           # $used is a HASHREF
     print $p->used('strict')     # true if code includes 'use strict'
 
@@ -182,16 +354,15 @@ of times they were used.
 If called with an argument, looks up the value of the argument in the
 hash and returns the number of times it was found during parsing.
 
-This is the prefered accessor.
+This is the preferred accessor.
 
-=cut
+=head3 used_in_eval
 
-sub used {
-    my $self=shift;
-    my $key=shift;
-    return $self->{found}{$key} if ($key);
-    return $self->{found};
-}
+Same as C<used>, except for considering in-eval-context only.
+
+=head3 used_out_of_eval
+
+Same as C<used>, except for considering NOT-in-eval-context only.
 
 =head3 string
 
@@ -202,13 +373,13 @@ C<$seperator> or using a blank space as a default;
 
 Module names are sorted by ascii value (i.e by C<sort>)
 
-=cut
+=head3 string_in_eval
 
-sub string {
-    my $self=shift;
-    my $sep=shift || ' ';
-    return join($sep,sort keys(%{$self->{found}}));
-}
+Same as C<string>, except for considering in-eval-context only.
+
+=head3 string_out_of_eval
+
+Same as C<string>, except for considering NOT-in-eval-context only.
 
 =head3 array
 
@@ -216,11 +387,13 @@ sub string {
 
 Returns an array of all used modules.
 
-=cut
+=head3 array_in_eval
 
-sub array {
-    return keys(%{shift->{found}})
-}
+Same as C<array>, except for considering in-eval-context only.
+
+=head3 array_out_of_eval
+
+Same as C<array>, except for considering NOT-in-eval-context only.
 
 =head3 arrayref
 
@@ -228,43 +401,18 @@ sub array {
 
 Returns a reference to an array of all used modules. Surprise!
 
-=cut
+=head3 arrayref_in_eval
 
-sub arrayref { 
-    my @a=shift->array;
-    return \@a if @a;
-    return;
-}
+Same as C<array_ref>, except for considering in-eval-context only.
+
+=head3 arrayref_out_of_eval
+
+Same as C<array_ref>, except for considering NOT-in-eval-context only.
 
 =head3 files
 
 Returns the number of files parsed by the parser object.
 
-=cut
-
-sub files {
-    return shift->{files};
-}
-
-# Internal Accessor Methods
-sub _add {
-    my $self=shift;
-    my $found=shift;
-    $self->{found}{$found}++;
-}
-
-sub _found {
-    return shift->{found}
-}
-
-sub _inc_files {
-    shift->{files}++
-}
-
-1;
-
-__END__
-
 =head1 RE-COMPILING THE GRAMMAR
 
 If - for some reasons - you need to alter the grammar, edit the file
@@ -284,21 +432,13 @@ Parse::RecDescent, Module::ScanDeps, Module::Info, Module::CPANTS::Analyse
 
 =head1 AUTHOR
 
-Thomas Klausner <domm@zsi.at>
+Thomas Klausner <domm@cpan.org>
 
-=head1 BUGS
+=head1 COPYRIGHT AND LICENSE
 
-Please report any bugs or feature requests to
-C<bug-module-extractuse@rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org>.  I will be notified, and then you'll automatically
-be notified of progress on your bug as I make changes.
+This software is copyright (c) 2012 by Thomas Klausner.
 
-=head1 LICENSE
-
-Module::ExtractUse is Copyright (c) 2003-2008, Thomas Klausner.
-
-You may use and distribute this module according to the same terms
-that Perl is distributed under.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
-
